##############################################################################
# 
# File            : tsiInit.tcl
# Author          : Gordon Baxter <gdb@vpsyc.psyc.nott.ac.uk>
# Created On      : 16 May 1996, 14:24:30
# 
# PURPOSE
# 
#   This is the initialization interface to the Tcl/TK Soar Interface (TSI),
#   which is designed to work with Soar 7.1 under Unix, Windows, and Macintosh
#
# Copyright 1997, Frank E. Ritter.
#
# $Id: tsiInit.tcl,v 8.2 1998/10/08 13:56:09 swallace Exp swallace $
# $Log: tsiInit.tcl,v $
# Revision 8.2  1998/10/08 13:56:09  swallace
# Released to John
#
# Revision 8.1  1998/09/18 16:42:34  swallace
# Works fine,, but dirty.  About to clean up.
#
# Revision 7.2  1998/08/21 12:11:44  swallace
# seems to work w/ remote agent proprioceptic feedback
#
# Revision 7.1  1998/06/23 13:50:01  swallace
# Works w/ eaters in Run Mode & Step Mode
# Works w/ CLIPS
#
# Revision 6.1  1998/06/02 19:01:43  swallace
# allows concurrent remote/local agents.
#
# Revision 4.1  1998/06/01 22:14:24  swallace
# works using tep on both sides.
#
# Revision 3.2  1998/05/28 20:18:39  swallace
# Ready to work on WME wrappers.
#
# Revision 3.1  1998/05/21 18:37:04  swallace
# Client/Server Interface allows info to be passed to client.
#
# Revision 2.1  1998/05/19 17:22:03  swallace
# *** empty log message ***
#
#
###############################################################################
### The Tcl/Tk Soar Interface, for Soar version 7.1
### Major contributors (in alphabetical order):
###    Gordon D. Baxter
###    Randolph M. Jones
###    Douglas J. Pearson
###    Frank E. Ritter
###    Karl B. Schwamb



# safety net in case of typos/misunderstandings
proc TSI {args} {eval tsi $args}

### Call "tsi" to start up the Tcl/Tk Soar Interface. 
###
### tsi [0|1] [option...]
###  
### The first optional argument, can be 0 or 1.  If 1, the control panel
### is hidden, otherwise it is displayed.
### 
### Additionally, a number of flags can be set, each of which has a specified
### domain.  These flags are:
###
###	-controlpanel	specifies the procedure which creates the 
###			controlpanel.
###	-x		the x location of the control panel
###	-y		the y location of the control panel
###	-mode		determines the socketio mode 
###			(one of: off, client, server)
###	-host		the default host, used if mode is set to 'client'
###	-port		the connection port, used if mode is 'client' or 
###			'server'
###
###	NOTE: procedures which are used as values to the -controlpanel flag
###	should return the name of frame to which socketio controls will
###	be added when they are needed.  If the top level frame is indicated
###	the return value should be {}, otherwise the frame will be indicated
###	with .<frame-name> (e.g. .socketio)
proc tsi {{hideController 0} args } {
   global interp_name interp_agentSocket tsi_version tsi_library \
       tcl_platform tsiConfig tsiSimulationState tsiSimulatorPath \
       tsiClicksPerSecond

   ## Make sure tk is loaded, and halt if it is not.

   if {[info commands tk]=={}} {
      puts {You are not running a version of Soar that supports the TSI. 
Run a version with TK, contact your system administrator, or email
soar-bugs@umich.edu for help.   

[Load aborted in tsi.tcl]}
      error "Cannot run TSI without the Tk package"
   }

   ### The tsi must be initialized from a non-soar interpreter
   foreach i [info loaded {}] {
      if {[lindex $i 1]=={Soar}} {
         tk_dialog .error Error {The TSI interface can only be initialized from the controlling interpreter.  This should be the top-level interpreter, and it should not have the Soar package loaded.} \
                   error 0 Dismiss
         error "Cannot start TSI from a Soar agent interpreter."
      }
   }
 


   set tsi_version 3.0alpha
   set tsiSimulatorPath [pwd]

   ### Read in the default TSI parameters
   ### These *must* exist, or the TSI will not run properly
   uplevel #0 [list source [file join $tsi_library tsi-defaults.tcl]]

   ### Read in user parameters if they exist
   if [file exists [file join [pwd] tsi-defaults.tcl]] {
      uplevel #0 [list source [file join [pwd] tsi-defaults.tcl]]
   }
 


    ### now check the additional arguments, if any
    if { [expr [llength $args] % 2] != 0 } { 
       error \
	   "Additional arguments to tsi must be defined with flags and values."
    }
    

    # By default, set the mode to off.
    set tsiConfig(mode) off
    set controlPanelProc makeTSIDefaultControlPanel
    set x $tsiConfig(ControlPanelX)
    set y $tsiConfig(ControlPanelY)

    foreach {key value} $args {
       switch -exact -- $key {
         -mode  { switch -exact $value {
	     client   { set tsiConfig(mode) client }
	     server   { set tsiConfig(mode) server }
	     off      { set tsiConfig(mode) off }
	     default  { error "Unrecognized mode '$value'" }
	     }
	 }
         -host         { set host $value }
         -port         { set port $port}
         -x            { set x $value }
         -y            { set y $value }
         -controlpanel { set controlPanelProc $value }
       }
   }


   set tsiSimulationState(running) 0
   
   if { $tsiConfig(calibrateClock) > 0 } {
      
      puts "Calibrating System Clock: $tsiConfig(calibrateClock) seconds please..."
      set ms [expr $tsiConfig(calibrateClock) * 1000]
      set begin [clock clicks]
      after $ms
      set end [clock clicks]
      set tsiClicksPerSecond [expr ($end - $begin)/$tsiConfig(calibrateClock)]
   }

   tsiInitInterpreter
  

   # SocketIO added 5/98	S.Wallace
   set sioTopFrame [$controlPanelProc $hideController $x $y]
   tsiSetControlPanelSIOMode $sioTopFrame

   puts "TSI ver. $tsi_version, expertise: $tsiConfig(expertise), \
debug: $tsiConfig(debug), Platform: $tcl_platform(platform)"

   if {$tsiConfig(debug) && [info exists env(DISPLAY)]} {
      global env
      puts "\nDisplay set to >>$env(DISPLAY)<<"
   }
}



proc tsiInitRemoteAgent {} {

    wm withdraw .    
    tsiInitInterpreter

}

### Call "tsiInitAgent" to initialize a Soar agent interpreter to use
### the TSI
proc tsiInitAgent {} {
   global tsi_library tsiConfig soar7km_aliases
    
   #if { $tsiConfig(sioDebug) > 3 } { puts "tsiInitAgent -- (begin)" }
   wm withdraw . 

   tsiInitInterpreter
 
   
   rename watch tsiInternalWatch
   rename help tsiInternalHelp
    
   if $tsiConfig(debug) {puts "\n\nPrinting watch\n\n"}
   if $tsiConfig(debug) {puts [tsiInternalWatch]}
   if $tsiConfig(debug) {puts "\n\nPrinted\n\n"}
    
   alias p2 print -depth 2
   alias d1 d 1
   alias 1 d 1
   alias 2 d 2
   alias 3 d 3
   alias 4 d 4
   alias 5 d 5
   alias 6 d 6
    alias 7 d 7
   alias 8 d 8
    alias 9 d 9


    if { ![info exists soar7km_aliases] } {
	### this should test a global var to see if they are already loaded.
	## Load the standard aliases
	if [catch {source [file join $tsi_library soar704km-aliases.tcl]} msg] {
	    puts "Failed to load Soar KM aliases: $msg"
	} else {
	    puts {Soar KM aliases loaded.  Type 'help alias' or 'alias' for more info.}
	}
    }   
   
    if [catch {makeSoarInteractionWindow $tsiConfig(AgentWindowX) \
		   $tsiConfig(AgentWindowY)} msg] {
	error "Failed to create Soar agent interaction window: $msg"
    }
    
    monitor -add before-init-soar cleanupMonitorWindows
    
    #if { $tsiConfig(sioDebug) > 3 } { puts "tsiInitAgent -- (end)" }

}


### Call tsiInitInterpreter to initialize use of the TSI from
### *any* interpreter (including the Control Panel).

### Any time a new interpreter is created, you have to give it the
### correct global auto_path value and then call this procedure to set
### all the other appropriate global variables.
proc tsiInitInterpreter {} {
   ### Do our "nicer" version of the "source" command
 
  rename source tsiInternalSource
 
   proc source {args} {
      global tsiSourceArgLast
 
      if {$args == {}} {
         puts "\[source $tsiSourceArgLast\]"
         ### We have to put the uplevel here because some procedures (like
         ### the package loader) depend on the loaded file sharing a variable
         ### with the calling procedure.  How's that for good programming
         ### style?
         uplevel 1 tsiInternalSource "$tsiSourceArgLast"
	
      } else {
	
          set tsiSourceArgLast $args
          uplevel 1 tsiInternalSource $args
	
      }
   }

}


proc tsiSetupAgentVars { agent } {
   global tsiClicksPerSecond

   $agent eval [list set tsiAgentStats(clicksPerSecond) $tsiClicksPerSecond]
   tsiResetAgentVars $agent
}

proc tsiResetAgentVars { agent } {

   $agent eval [list set tsiAgentStats(inputTime) 0]
   $agent eval [list set tsiAgentStats(outputTime) 0]
   $agent eval [list set tsiAgentStats(inputCycles) 0]
   $agent eval [list set tsiAgentStats(outputCycles) 0]
}






proc localAgentAddWME { agent objectID attribute value } {
   global tsiConfig

   if { $tsiConfig(sioDebug) > 4 } {
      puts "Adding WME to local agent: ($agent $objectID $attribute $value)"
   }

 
   return [$agent eval "add-wme $objectID $attribute $value"]

}






proc localAgentRemoveWME { agent timeTag } {
   return [$agent eval "remove-wme $timeTag"]
}

   

proc ChangeWME { oldID newPoint } {
}




#
# ldelete
#
# This function is a little function copied from the Tcl/TK book.  It deletes
#   an item from a list based on value.
#

proc ldelete { list value } {
	 set ix [lsearch -exact $list $value]
	 if {$ix >= 0} {
		  return [lreplace $list $ix $ix]
	 } else {
		  return $list
	 }
}

    
    